from java.lang import Object


class Animator(Object):
    def cancel(self):
        # type: () -> bool
        pass

    def pause(self):
        # type: () -> None
        pass

    def resume(self):
        # type: () -> None
        pass
