__all__ = ["ScheduleModel"]

from com.palantir.ptoss.cinch.core import DefaultBindableModel


class ScheduleModel(DefaultBindableModel):
    def deselectDayOfWeek(self, day):
        pass

    def getEveryHour(self):
        pass

    def getHour(self):
        pass

    def getMinute(self):
        pass

    def getSelectedDays(self):
        pass

    def isAlldays(self):
        pass

    def isDaySelected(self, day):
        pass

    def isEveryHourEnabled(self):
        pass

    def isEveryMinuteEnabled(self):
        pass

    def isHourEnabled(self):
        pass

    def isMinuteEnabled(self):
        pass

    def selectDayOfWeek(self, day):
        pass

    def setAllDays(self, allDays):
        pass

    def setEveryHour(self, everyHour):
        pass

    def setHour(self, hour):
        pass

    def setHourEnabled(self, hourEnabled):
        pass

    def setMinute(self, minute):
        pass

    def setMinuteEnabled(self, minuteEnabled):
        pass

    def setSelectedDays(self, selectedDays):
        pass

    def toCronString(self):
        pass
