"""The Ignition scripting API, which is available under the module name
"system", is full of functions that are useful when designing projects
in Ignition. From running database queries, manipulating components, to
exporting data, scripting functions can help. Some of these functions
only work in the Gateway scope, and other only work in the Client scope,
while the rest will work in any scope.

Additional information on scripting Ignition can be found in the
Scripting section.
"""
